@@ -25,12 +25,12 @@
 #include "mp.h"
 
 void register_init();
-void register_set(int index, int value[MP_SIZE]);
-void register_get(int index, int value[MP_SIZE]);
+void register_set(int index, MPNumber *value);
+void register_get(int index, MPNumber *value);
 
-void constant_set(int index, const char *name, int value[MP_SIZE]);
+void constant_set(int index, const char *name, MPNumber *value);
 const char *constant_get_name(int index);
-const int *constant_get_value(int index);
+const MPNumber *constant_get_value(int index);
 
 void function_set(int index, const char *name, const char *value);
 const char *function_get_name(int index);
